#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_bluetooth.h"
#include "user.h"
#include "timer.h"
#include "xprintf.h"

#define PIN_CODE_DEFAULT	"0000"
#define BOARD_SBDBT         1

// Define the baud rate constants UART1
#define	UARTPARAM1			UARTPARAM_N81			//data=8bit parity=none stopbit=1bit

#define BAUDRATE1       	115200
#define	USE_RFCOMM_BAUDRATE	0						//0:baud rate as fixed 1:baud rate as RFCOMM parameter from Master
#define BRG_DIV1        	4
#define BRGH1           	1
// Define Hardware Flow Control UART1 (0:off 1:on)
#define	UART1_HW_FLOW		0
#define	UART1_TX_POLARITY	0						//MMɐ 0:ʏ 1:]
#define	UART1_RX_POLARITY	0						//MMɐ 0:ʏ 1:]

// Define the baud rate constants UART2 (Debug)
#define BAUDRATE2       	115200
#define BRG_DIV2        	4
#define BRGH2           	1

// Define LED1 communication indicator on time(ms)
#define	LED1_ON_COUNT		10
#define	USE_RSSI			1						//0:blink rate as fixed 1:blink rate as RSSI (fast:lowRSSI slow:highRSSI)



#if BOARD_SBDBT
#define LOCAL_NAME			"SBDBT"
#define	ICSDEF				ICS_PGx1
#endif

#if	__DEBUG
//disable watchdog timer
#define	FWDTENDEF			FWDTEN_OFF
#define	RestartWatchdog()
#define STO_SERIAL			1   //0:STO as I/O  1:STO as serial output
#else
//enable watchdog timer
#define	FWDTENDEF			FWDTEN_ON
#define	RestartWatchdog()	ClrWdt()
#define STO_SERIAL			0   //0:STO as I/O  1:STO as serial output
#endif

//
//for SBDBT board
//
// Configure LED1 - put on pin 14 (RB14)
#define	led1_setup()		(TRISBbits.TRISB14=0)
#define	led1_on()			(LATBbits.LATB14=0)	//ON
#define	led1_off()			(LATBbits.LATB14=1)	//OFF

// Configure LED2 - put on pin 15 (RB15)
#define	led2_setup()		(TRISBbits.TRISB15=0)
#define	led2_on()			(LATBbits.LATB15=0)	//ON
#define	led2_off()			(LATBbits.LATB15=1)	//OFF

//scl:SBDBT PIN8:RP23/PMA0/CN17/RC7
#define	scl_setup()			(TRISCbits.TRISC7=1,CNPU2bits.CN17PUE=1)	//input,pull-up
#define	ddr_set_scl()		(TRISCbits.TRISC7=0)
#define	ddr_reset_scl()		(TRISCbits.TRISC7=1)
#define	scl_h_mac()			(LATCbits.LATC7=1)
#define	scl_l_mac()			(LATCbits.LATC7=0)
#define	get_scl()			(PORTCbits.RC7)

//sda:SBDBT PIN9:RP24/PMA5/CN20/RC8
#define	sda_setup()			(TRISCbits.TRISC8=1,CNPU2bits.CN20PUE=1)	//input,pull-up
#define	ddr_set_sda()		(TRISCbits.TRISC8=0)	//output
#define	ddr_reset_sda()		(TRISCbits.TRISC8=1)	//input
#define	sda_h()				(LATCbits.LATC8=1)
#define	sda_l()				(LATCbits.LATC8=0)
#define	get_sda()			(PORTCbits.RC8)



#define	u1rx_setup()
#define u1tx_setup()
#define	u1cts_setup()
#define	u1rts_setup()
#define	u1rts_on()
#define	u1rts_off()

#if	STO_SERIAL
// Configure console OUT(U2TX) - put on pin 1 (RP9)
#define	sto_setup()			(RPOR4bits.RP9R = 5)
#define	sto_l()
#define	sto_h()
#else
// Configure Status OUT - put on pin 1 (RB9)
#define	sto_setup()		(TRISBbits.TRISB9=0)
#define	sto_l()			(LATBbits.LATB9=0)
#define	sto_h()			(LATBbits.LATB9=1)
#endif

// no association out on this board
#define	assoc_setup()
#define	assoc_l()
#define	assoc_h()

// no on out on this board
#define	on_setup()

#define	pairen_setup()
#define	get_pairen()		(1)


#define Nop()    __builtin_nop()
#define ClrWdt() {__asm__ volatile ("clrwdt");}
#define Sleep()  {__asm__ volatile ("pwrsav #0");}
#define Idle()   {__asm__ volatile ("pwrsav #1");}

extern void (*usb_tick_handler)(void);
void	event_bluetooth_rxEvent_done(WORD size);
void	event_bluetooth_rxAcl_done(WORD size);

int init_mma8451q(void);
int read_mma8451q(short *x,short *y,short *z);
